import { NetplayInput, NetplayPlayer, NetplayState, JSONValue } from "../types";
declare class NetplayHistory<TInput extends NetplayInput<TInput>> {
    /**
     * The frame number that this history entry represents.
     */
    frame: number;
    /**
     * The serialized state of the game at this frame.
     */
    state: JSONValue;
    /**
     * These inputs represent the set of inputs that produced this state
     * from the previous state. So history[n].state = history[n - 1].state.tick(history)
     */
    inputs: Map<NetplayPlayer, {
        input: TInput;
        isPrediction: boolean;
    }>;
    constructor(frame: number, state: JSONValue, inputs: Map<NetplayPlayer, {
        input: TInput;
        isPrediction: boolean;
    }>);
    isPlayerInputPredicted(player: NetplayPlayer): boolean;
    anyInputPredicted(): boolean;
    allInputsSynced(): boolean;
}
export declare class NetplayManager<TState extends NetplayState<TInput>, TInput extends NetplayInput<TInput>> {
    /**
     * The rollback history buffer.
     */
    history: Array<NetplayHistory<TInput>>;
    /**
     * The max number of frames that we can predict ahead before we have to stall.
     */
    maxPredictedFrames: number;
    /**
     * Inputs from other players that have already arrived, but have not been
     * applied due to our simulation being behind.
     */
    future: Map<NetplayPlayer, Array<{
        frame: number;
        input: TInput;
    }>>;
    highestFrameReceived: Map<NetplayPlayer, number>;
    /**
     * Whether or not we are the host of this match. The host is responsible for
     * sending our authoritative state updates.
     */
    isServer: boolean;
    onStateSync(frame: number, state: JSONValue): void;
    onRemoteInput(frame: number, player: NetplayPlayer, input: TInput): void;
    broadcastInput: (frame: number, input: TInput) => void;
    broadcastState?: (frame: number, state: JSONValue) => void;
    pingMeasure: any;
    timestep: number;
    state: TState;
    constructor(isServer: boolean, initialState: TState, initialInputs: Map<NetplayPlayer, TInput>, maxPredictedFrames: number, pingMeasure: any, timestep: number, broadcastInput: (frame: number, input: TInput) => void, broadcastState?: (frame: number, state: JSONValue) => void);
    currentFrame(): number;
    largestFutureSize(): number;
    predictedFrames(): number;
    shouldStall(): boolean;
    tick(localInput: TInput): void;
    /**
     * Internally, we store inputs with a flag indicating whether or not that input is
     * a prediction. Before sending that to the state, we need to remove the prediction
     * flags, since the game logic doesn't care.
     */
    getStateInputs(inputs: Map<NetplayPlayer, {
        input: TInput;
        isPrediction: boolean;
    }>): Map<NetplayPlayer, TInput>;
}
export {};
