export declare type JSONPrimitive = string | number | boolean | null;
export declare type JSONValue = JSONPrimitive | JSONObject | JSONArray;
export declare type JSONObject = {
    [member: string]: JSONValue;
};
export interface JSONArray extends Array<JSONValue> {
}
export declare abstract class NetplayState<TInput extends NetplayInput<TInput>> {
    abstract tick(playerInputs: Map<NetplayPlayer, TInput>): void;
    /**
     * By default, use the auto serializer.
     */
    serialize(): JSONValue;
    /**
     * By default, use the auto deserializer.
     */
    deserialize(value: JSONValue): void;
}
export declare abstract class NetplayInput<TInput extends NetplayInput<TInput>> {
    /**
     * By default, the prediction is to just use the same value.
     */
    predictNext(): TInput;
    /**
     * By default, use the auto serializer.
     */
    serialize(): JSONValue;
    /**
     * By default, use the auto deserializer.
     */
    deserialize(value: JSONValue): void;
}
export interface NetplayPlayer {
    isLocalPlayer(): boolean;
    isRemotePlayer(): boolean;
    isServer(): boolean;
    isClient(): boolean;
    getID(): number;
}
export interface GameType<TState, TInput> {
    /**
     * Given a list of players, return the initial game state.
     */
    constructInitialState(players: Array<NetplayPlayer>): TState;
    /**
     * Construct a new input object with a default value. A new object
     * needs to be constructed, since serialized values will be copied into this.
     */
    constructDefaultInput(): TInput;
    /**
     * The game simulation timestep, in milliseconds.
     */
    timestep: number;
    canvasWidth: number;
    canvasHeight: number;
    draw(state: TState, canvas: HTMLCanvasElement, ctx: CanvasRenderingContext2D): any;
    getInputReader(document: HTMLDocument, canvas: HTMLCanvasElement): () => TInput;
}
